/*
 * mini-cp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  v3
 * as published by the Free Software Foundation.
 *
 * mini-cp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 * See the GNU Lesser General Public License  for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with mini-cp. If not, see http://www.gnu.org/licenses/lgpl-3.0.en.html
 *
 * Copyright (c)  2017. by Laurent Michel, Pierre Schaus, Pascal Van Hentenryck
 */

package minicp;

import minicp.engine.core.IntVar;
import minicp.engine.core.Solver;
import minicp.search.DFSearch;
import minicp.search.Objective;
import minicp.search.SearchStatistics;
import minicp.util.io.InputReader;

import static minicp.cp.BranchingScheme.firstFail;
import static minicp.cp.Factory.*;

public class QAP {

    public static void main(String[] args) {

        // ---- read the instance -----



        int n = 12;
        // Weights
        int[][] w = new int[][] {
                {0,90,10,23,43,0,0,0,0,0,0,0},
                {90,0,0,0,0,88,0,0,0,0,0,0},
                {10,0,0,0,0,0,26,16,0,0,0,0},
                {23,0,0,0,0,0,0,0,0,0,0,0},
                {43,0,0,0,0,0,0,0,0,0,0,0},
                {0,88,0,0,0,0,0,0,1,0,0,0},
                {0,0,26,0,0,0,0,0,0,0,0,0},
                {0,0,16,0,0,0,0,0,0,96,0,0},
                {0,0,0,0,0,1,0,0,0,0,29,0},
                {0,0,0,0,0,0,0,96,0,0,0,37},
                {0,0,0,0,0,0,0,0,29,0,0,0},
                {0,0,0,0,0,0,0,0,0,37,0,0}
        };
        // Distance
        int maxDist = 0;
        int[][] d = new int[][] {
                {0,36,54,26,59,72,9,34,79,17,46,95},
                {36,0,73,35,90,58,30,78,35,44,79,36},
                {54,73,0,21,10,97,58,66,69,61,54,63},
                {26,35,21,0,93,12,46,40,37,48,68,85},
                {59,90,10,93,0,64,5,29,76,16,5,76},
                {72,58,97,12,64,0,96,55,38,54,0,34},
                {9,30,58,46,5,96,0,83,35,11,56,37},
                {34,78,66,40,29,55,83,0,44,12,15,80},
                {79,35,69,37,76,38,35,44,0,64,39,33},
                {17,44,61,48,16,54,11,12,64,0,70,86},
                {46,79,54,68,5,0,56,15,39,70,0,18},
                {95,36,63,85,76,34,37,80,33,86,18,0}
        };

        // ----- build the model ---

        Solver cp = makeSolver();
        IntVar[] x = makeIntVarArray(cp, n, n);

        cp.post(allDifferent(x));


        // build the objective function
        IntVar[] weightedDist = new IntVar[n * n];
        for (int k = 0, i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                weightedDist[k] = mul(element(d, x[i], x[j]), w[i][j]);
                k++;
            }
        }
        IntVar totCost = sum(weightedDist);
        Objective obj = cp.minimize(totCost);

        DFSearch dfs = makeDfs(cp, firstFail(x));

        dfs.onSolution(() -> System.out.println("obj = " + totCost.min()));


        long t0 = System.currentTimeMillis();

        SearchStatistics stats = dfs.optimize(obj);

        long t1 = System.currentTimeMillis();



        System.out.println("time(ms): " + (t1 - t0));
        System.out.println(stats);


    }
}
