import org.rogach.scallop._

import minicp._
import oscar._
import choco._

class Conf(arguments: Seq[String]) extends ScallopConf(arguments) {
  val solver = choice(Seq("choco","minicp","oscar"),required = true, descr = "solver")
  val problem = choice(Seq("magic","square","queens","qap","tsp"),required = true, descr = "problem")
  verify()
}

object Main {
  def main(args: Array[String]) {

    val conf = new Conf(args) // Note: This line also works for "object Main extends App"

    val descr = Map[String, String]("square" -> "5x5 Magic Square Problem naive model, search for first 10000 solutions", "magic" -> "magic series problem of length 300, naive model, search for all solutions", "tsp" -> "TSP problem solved optimally instance gr17 https://people.sc.fsu.edu/~jburkardt/datasets/tsp/gr17_d.txt", "qap" -> "Quadratic Assignment Problem, instance 12x12 solved optimally", "queens" -> "81 queens, first fail min value, search for first solution")

    println(descr(conf.problem.getOrElse("problem")) + " with solver " + conf.solver.getOrElse("solver"))

    conf.solver.getOrElse("choco") match {
      case "choco" => {
        println("Choco 4.0.2")
        conf.problem.getOrElse("magic") match {
          case "magic" =>
            choco.MagicSerie.main(Array.empty);
          case "square" =>
            choco.MagicSquare.main(Array.empty);
          case "queens" =>
            choco.NQueens.main(Array.empty);
          case "qap" =>
            choco.QAP.main(Array.empty);
          case "tsp" =>
            choco.TSP.main(Array.empty);
        }
      }
      case "minicp" => {
        println("Mini-CP 1.0.0")
        conf.problem.getOrElse("magic") match {
          case "magic" =>
            minicp.MagicSerie.main(Array.empty);
          case "square" =>
            minicp.MagicSquare.main(Array.empty);
          case "queens" =>
            minicp.NQueens.main(Array.empty);
          case "qap" =>
            minicp.QAP.main(Array.empty);
          case "tsp" =>
            minicp.TSP.main(Array.empty);
        }

      }
      case "oscar" => {
        println("OscaR 4.0.0")
        conf.problem.getOrElse("magic") match {
          case "magic" =>
            oscar.MagicSerie.main(Array.empty);
          case "square" =>
            oscar.MagicSquare.main(Array.empty);
          case "queens" =>
            oscar.NQueens.main(Array.empty);
          case "qap" =>
            oscar.QAP.main(Array.empty);
          case "tsp" =>
            oscar.TSP.main(Array.empty);
        }
      }
    }



    /*
    val bench = new Bench(IO.read(conf.in()),conf.volume(),conf.price(),conf.gap(),Seq())
    val ids = SolveLS.solve(bench,if (conf.time.isDefined)  conf.time() else 5,if (conf.verbose.isDefined) conf.verbose() else false)
    println("solution verification:")
    println(bench.verify(ids))
    println("ids:"+ids.mkString(","))*/


  }
}