package oscar

/** *****************************************************************************
  * OscaR is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
  * the Free Software Foundation, either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * OscaR is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Lesser General Public License  for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License along with OscaR.
  * If not, see http://www.gnu.org/licenses/lgpl-3.0.en.html
  * *****************************************************************************/

import oscar.cp._

import scala.math._

/**
  * Magic square in Oscar.
  *
  * @author Hakan Kjellerstrand hakank@gmail.com
  *         http://www.hakank.org/oscar/
  */
object MagicSquare extends CPModel with App {
  val n = 5;
  val n2 = n * n

  //
  // variables
  //
  val x: Array[Array[CPIntVar]] = Array.fill(n, n)(CPIntVar(1 to n2))
  val x_t: Array[Array[CPIntVar]] = x.transpose
  // val total = CPIntVar(cp, 1 to n*n*n)
  val total = (n * (n * n + 1) / 2)
  //
  // constraints
  //

  //add(allDifferent(x.flatten), Strong)

  val xflat = x.flatten.toArray
  for (i <- 0 until xflat.length; j <- i+1 until xflat.length) {
    solver.add(xflat(i) !== xflat(j))
  }

  // rows and columns
  for (i <- 0 until n) {
    solver.add(sum(x(i)) === total)
    solver.add(sum(x_t(i)) === total)
  }
  // diagonals
  add(sum(for (i <- 0 until n) yield x(i)(i)) === total)
  add(sum(for (i <- 0 until n) yield x(i)(n - i - 1)) === total)
  // symmetry breaking
  add(x(0)(0) < x(0)(n - 1))
  add(x(0)(n - 1) < x(n - 1)(0))
  add(x(0)(0) < x(n - 1)(n - 1))
  search {
    binaryFirstFail(x.flatten.toSeq, _.min)
  }

  println(start(10000))
}
