package oscar

import oscar.cp._
import oscar.util._

import scala.io.Source

/**
 * Quadratic Assignment Problem:
 * There are a set of n facilities and a set of n locations.
 * For each pair of locations, a distance is specified and
 * for each pair of facilities a weight or flow is specified
 * (e.g., the amount of supplies transported between the two facilities).
 * The problem is to assign all facilities to different locations
 * with the goal of minimizing the sum of the distances multiplied by the corresponding flows.
 *
 * @author Pierre Schaus pschaus@gmail.com
 */
object QAP extends CPModel with App {

  // Read the data
  val n = 12
  val N = 0 until n

  // Weights
  val w = Array(
    Array(0,90,10,23,43,0,0,0,0,0,0,0),
    Array(90,0,0,0,0,88,0,0,0,0,0,0),
    Array(10,0,0,0,0,0,26,16,0,0,0,0),
    Array(23,0,0,0,0,0,0,0,0,0,0,0),
    Array(43,0,0,0,0,0,0,0,0,0,0,0),
    Array(0,88,0,0,0,0,0,0,1,0,0,0),
    Array(0,0,26,0,0,0,0,0,0,0,0,0),
    Array(0,0,16,0,0,0,0,0,0,96,0,0),
    Array(0,0,0,0,0,1,0,0,0,0,29,0),
    Array(0,0,0,0,0,0,0,96,0,0,0,37),
    Array(0,0,0,0,0,0,0,0,29,0,0,0),
    Array(0,0,0,0,0,0,0,0,0,37,0,0)
  );
  // Distance
  val d = Array(
    Array(0,36,54,26,59,72,9,34,79,17,46,95),
    Array(36,0,73,35,90,58,30,78,35,44,79,36),
    Array(54,73,0,21,10,97,58,66,69,61,54,63),
    Array(26,35,21,0,93,12,46,40,37,48,68,85),
    Array(59,90,10,93,0,64,5,29,76,16,5,76),
    Array(72,58,97,12,64,0,96,55,38,54,0,34),
    Array(9,30,58,46,5,96,0,83,35,11,56,37),
    Array(34,78,66,40,29,55,83,0,44,12,15,80),
    Array(79,35,69,37,76,38,35,44,0,64,39,33),
    Array(17,44,61,48,16,54,11,12,64,0,70,86),
    Array(46,79,54,68,5,0,56,15,39,70,0,18),
    Array(95,36,63,85,76,34,37,80,33,86,18,0)
  );


  // for each facilities, the location chosen for it
  val x = N map (v => CPIntVar(0 until n))

  //add(allDifferent(x), Strong)
  for (i <- 0 until n; j <- i+1 until n) {
    add(x(i) !== x(j))
  }
  var obj = sum(N, N)((i, j) => d(x(i))(x(j)) * w(i)(j))

  minimize(obj)

  onSolution { println("obj = " + obj.value) }

  search(binaryFirstFailIdx(x,i => x(i).min))

  val stats = start()
  println(stats)
}
