/*******************************************************************************
  * OscaR is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
  * the Free Software Foundation, either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * OscaR is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Lesser General Public License  for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License along with OscaR.
  * If not, see http://www.gnu.org/licenses/lgpl-3.0.en.html
  ******************************************************************************/

package oscar

import oscar.cp._
import oscar.cp.constraints.{Circuit, Sum}
import oscar.util._

import scala.io.Source

/**
 *
 * tsp model: given a distance matrix between 20 cities,
 *            find the shortest tour visiting each city exactly once.
 *
 * @author Pierre Schaus pschaus@gmail.com
 */
object TSP extends CPModel with App {

  solver.silent = true

  val n = 17
  val Cities = 0 until n

  val distMatrix = Array[Array[Int]](Array(0, 633, 257, 91, 412, 150, 80, 134, 259, 505, 353, 324, 70, 211, 268, 246, 121), Array(633, 0, 390, 661, 227, 488, 572, 530, 555, 289, 282, 638, 567, 466, 420, 745, 518), Array(257, 390, 0, 228, 169, 112, 196, 154, 372, 262, 110, 437, 191, 74, 53, 472, 142), Array(91, 661, 228, 0, 383, 120, 77, 105, 175, 476, 324, 240, 27, 182, 239, 237, 84), Array(412, 227, 169, 383, 0, 267, 351, 309, 338, 196, 61, 421, 346, 243, 199, 528, 297), Array(150, 488, 112, 120, 267, 0, 63, 34, 264, 360, 208, 329, 83, 105, 123, 364, 35), Array(80, 572, 196, 77, 351, 63, 0, 29, 232, 444, 292, 297, 47, 150, 207, 332, 29), Array(134, 530, 154, 105, 309, 34, 29, 0, 249, 402, 250, 314, 68, 108, 165, 349, 36), Array(259, 555, 372, 175, 338, 264, 232, 249, 0, 495, 352, 95, 189, 326, 383, 202, 236), Array(505, 289, 262, 476, 196, 360, 444, 402, 495, 0, 154, 578, 439, 336, 240, 685, 390), Array(353, 282, 110, 324, 61, 208, 292, 250, 352, 154, 0, 435, 287, 184, 140, 542, 238), Array(324, 638, 437, 240, 421, 329, 297, 314, 95, 578, 435, 0, 254, 391, 448, 157, 301), Array(70, 567, 191, 27, 346, 83, 47, 68, 189, 439, 287, 254, 0, 145, 202, 289, 55), Array(211, 466, 74, 182, 243, 105, 150, 108, 326, 336, 184, 391, 145, 0, 57, 426, 96), Array(268, 420, 53, 239, 199, 123, 207, 165, 383, 240, 140, 448, 202, 57, 0, 483, 153), Array(246, 745, 472, 237, 528, 364, 332, 349, 202, 685, 542, 157, 289, 426, 483, 0, 336), Array(121, 518, 142, 84, 297, 35, 29, 36, 236, 390, 238, 301, 55, 96, 153, 336, 0))


  //solver.silent = true
  
  // Successors
  val succ = Array.tabulate(n)(_ => CPIntVar.sparse(0, n))
  
  // Total distance
  val dist = CPIntVar(0, distMatrix.flatten.sum)


  for (i <- 0 until n; j <- i+1 until n) {
    add(succ(i) !== succ(j))
  }


  add(new Circuit(succ, false), Weak)

  add(new Sum((0 until n).map(i => distMatrix(i)(succ(i))), dist))


  onSolution(println("totDist = "+dist.value))
  minimize(dist)

  search(binaryFirstFailIdx(succ,i => succ(i).min))
  
  println(start())
}
